# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['cp2k_input_tools']

package_data = \
{'': ['*']}

install_requires = \
['pint>=0.9,<0.10', 'transitions>=0.7.0,<0.8.0']

extras_require = \
{'yaml': ['pyyaml>=5.1.2,<6.0.0']}

entry_points = \
{'console_scripts': ['cp2klint = cp2k_input_tools.cli:cp2klint',
                     'fromcp2k = cp2k_input_tools.cli:fromcp2k',
                     'tocp2k = cp2k_input_tools.cli:tocp2k']}

setup_kwargs = {
    'name': 'cp2k-input-tools',
    'version': '0.2.0',
    'description': 'Python tools to handle CP2K input files',
    'long_description': '# cp2k-input-tools\n\n[![Build Status](https://travis-ci.com/dev-zero/cp2k-input-tools.svg?branch=develop)](https://travis-ci.com/dev-zero/cp2k-input-tools) [![codecov](https://codecov.io/gh/dev-zero/cp2k-input-tools/branch/develop/graph/badge.svg)](https://codecov.io/gh/dev-zero/cp2k-input-tools)\n\nFully validating pure-python CP2K input file parsers including preprocessing capabilities\n\nAvailable commands:\n\n* `cp2klint` .. a CP2K input file linter\n* `fromcp2k` .. create a (almost simplified) JSON or YAML configuration file from a CP2K input file (includes validation)\n* `tocp2k` .. convert a (simplified) JSON or YAML configuration back to CP2K\'s input file format (includes validation)\n\nFor a description of the JSON/YAML formats used, see below.\n\n## Requirements\n\n* Python 3.6+\n* https://pypi.org/project/transitions/\n* optional: https://pypi.org/project/PyYAML/\n\nFor development: https://poetry.eustace.io/\n\n## Idea\n\n* have a pure-python CP2K input file linter with proper syntax error reporting (context, etc.)\n* a final & complete restart file parser\n* basis for an AiiDA CP2K project importer\n* testbed for alternative import formats (YAML, JSON) for CP2K\n* possible testbed for a re-implementation of the CP2K input parser itself\n\n## TODOs\n\n* parser: improve error reporting with context\n* preprocessor: don\'t lose original context when interpolating variables\n* parser: parsing the XML is slow (easily 70% of the time), pickle or generate Python code directly instead and keep XML parsing as fallback\n* parser: maybe generate AST using an emitting (`yield`) parser for more flexibility\n\n# The CP2K JSON and YAML formats\n\nA reference to the CP2K input format can be found here: https://manual.cp2k.org/\n\n## Canonical format\n\nFor everything except the pre-processor capabilities (`@IF/@ENDIF/$var/@SET`) there is a canonical one-to-one mapping of the CP2K input format to either JSON or YAML:\n\n* repeatable sections are mapped to dictionaries\n* keywords or subsections are key/value entries in sections\n* all repeatable elements (sections and keywords) are mapped to lists of their respective mapped datatype\n* section parameters are mapped to a special key named `_`\n* default section keywords are mapped to a special key name `*`\n* sections in JSON or YAML must be prefixed to avoid double definition of a key in case of same name for a section and a keyword (like the `POTENTIAL` in `KIND`), to avoid quotation marks, instead of CP2K\'s `&` we are using the `+`\n* keyword values are mapped based on their datatypes: a list of values is always mapped to a list of their respective datatypes\n\nThe following example input:\n\n```\n&GLOBAL\n   PRINT_LEVEL MEDIUM\n   PROJECT test\n   RUN_TYPE ENERGY\n&END GLOBAL\n&FORCE_EVAL\n   METHOD Quickstep\n   &DFT\n      BASIS_SET_FILE_NAME "./BASIS_SETS"\n      POTENTIAL_FILE_NAME ./POTENTIALS\n      &XC\n         &XC_FUNCTIONAL PBE\n         &END XC_FUNCTIONAL\n      &END XC\n   &END DFT\n   &SUBSYS\n      &CELL\n         A [angstrom] 4.07419 0.0 0.0\n         B [angstrom] 2.037095 3.52835204 0.0\n         C [angstrom] 2.037095 1.17611735 3.32656221\n         PERIODIC XYZ\n      &END CELL\n      &KIND Ge\n         ELEMENT Ge\n         POTENTIAL ALL-q32\n         BASIS_SET ORB pob-TZVP\n      &END KIND\n      &TOPOLOGY\n         COORD_FILE ./struct.xyz\n         COORD_FILE_FORMAT XYZ\n      &END TOPOLOGY\n   &END SUBSYS\n&END FORCE_EVAL\n```\n\nwould generate the (canonical) JSON:\n\n```json\n{\n  "+global": {\n    "print_level": "medium",\n    "project_name": "test",\n    "run_type": "energy"\n  },\n  "+force_eval": [\n    {\n      "method": "quickstep",\n      "+DFT": {\n        "basis_set_file_name": [\n          "./BASIS_SETS"\n        ],\n        "potential_file_name": "./POTENTIALS"\n      },\n      "+XC": {\n        "+xc_functional": {\n          "_": "PBE"\n        }\n      },\n      "+subsys": {\n        "cell": {\n          "A": [ 4.07419, 0, 0 ],\n          "B": [ 2.037095, 3.52835204, 0 ],\n          "C": [ 2.037095, 1.17611735, 3.32656221 ],\n          "periodic": "XYZ"\n        },\n        "+kind": [\n          {\n            "_": "Ge",\n            "element": "Ge",\n            "potential": "ALL-q32",\n            "basis_set": [\n              [ "ORB", "pob-TZVP" ]\n            ]\n          }\n          ],\n        "+topology": {\n          "coord_file_name": "./struct.xyz",\n          "coord_file_format": "XYZ"\n        }\n      }\n    }\n  ]\n}\n```\n\n*Caveats*:\n\n* the full input format needs be known and is being loaded from a bundled `cp2k_input.xml`\n* the YAML/JSON is quiet verbose and one has to know exactly which keywords can be repeated\n\nWhile there is no solution to remedy the first caveat, the second can be solved with the simplified output format\n\n## Simplified format\n\nStill based on the canonical format the simplified format relaxes some of the rules\n\n1. a section must only be prefixed with a `+` if a keyword with the same name is present at the same time in the same section (since we can figure out whether the user wanted to specify the section or the keyword by inspecting the value for the key: `dict` for a section)\n2. if a repeated keyword or section contains only one entry, the list can be omitted (in case of ambiguity priority is given to multiple values per keyword rather than keyword repetition)\n3. sections with default parameters can be formulated as dictionaries, as long as the default parameter values are unique and do not match section keyword or subsection names\n\nthe example from before in the simplified format:\n\n```json\n{\n  "global": {\n    "print_level": "medium",\n    "project_name": "test",\n    "run_type": "energy"\n  },\n  "force_eval": {\n    "method": "quickstep",\n    "DFT": {\n      "basis_set_file_name": "./BASIS_SETS",\n      "potential_file_name": "./POTENTIALS"\n    },\n    "xc": {\n      "xc_functional": {\n        "_": "PBE"\n      }\n    },\n    "subsys": {\n      "cell": {\n        "A": [ 4.07419, 0, 0 ],\n        "B": [ 2.037095, 3.52835204, 0 ],\n        "C": [ 2.037095, 1.17611735, 3.32656221 ],\n        "periodic": "XYZ"\n      },\n      "kind": {\n        "_": "Ge",\n        "element": "Ge",\n        "potential": "ALL-q32",\n        "basis_set": [ "ORB", "pob-TZVP" ]\n      },\n      "topology": {\n        "coord_file_name": "./struct.xyz",\n        "coord_file_format": "XYZ"\n      }\n    }\n  }\n}\n```\n\nor in YAML (with simplification rule #3 applied):\n\n```yaml\nglobal:\n  print_level: medium\n  project_name: test\n  run_type: energy\nforce_eval:\n  DFT:\n    basis_set_file_name: ./BASIS_SETS\n    potential_file_name: ./POTENTIALS\n  XC:\n    xc_functional:\n      _: PBE  # this can NOT be simplified since PBE could also be a subsection of xc_functional\n  method: quickstep\n  subsys:\n    cell:\n      A: [ 4.07419, 0.0, 0.0]\n      B: [ 2.037095, 3.52835204, 0.0]\n      C: [ 2.037095, 1.17611735, 3.32656221]\n      periodic: XYZ\n    kind:\n      Ge:\n        basis_set: [ORB, pob-TZVP]\n        element: Ge\n        potential: ALL-q32\n    topology:\n      coord_file_format: XYZ\n      coord_file_name: ./struct.xyz\n```\n',
    'author': 'Tiziano Müller',
    'author_email': 'tiziano.mueller@chem.uzh.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
